type BufferEncoding = 'base64' | 'hex' | 'latin1' | 'utf8';
type BufferEncodingNonstandard = 'binary' | 'json';
type EncodingAcronym = 'json' | 'utf8';
type MethodCase<T extends string> = T extends EncodingAcronym ? Uppercase<T> : Capitalize<T>;
type EncoderMethod<T, K extends string> = Record<`to${MethodCase<K>}`, () => T>;
export type BinaryStringEncoding = BufferEncoding | BufferEncodingNonstandard;
export type BinaryTranscoder = EncoderMethod<number[], 'array'> & EncoderMethod<number, 'number'> & EncoderMethod<Uint8Array, 'uInt8Array'> & EncoderMethod<string, BinaryStringEncoding>;
export default function transcode(param: number | number[] | Uint8Array | {
    encoding: BinaryStringEncoding;
    text: string;
}): BinaryTranscoder;
export declare const fromBase64: (text: string) => BinaryTranscoder;
export declare const fromBinary: (text: string) => BinaryTranscoder;
export declare const fromHex: (text: string) => BinaryTranscoder;
export declare const fromJSON: (text: string) => BinaryTranscoder;
export declare const fromLatin1: (text: string) => BinaryTranscoder;
export {};
